@@ -37,16 +37,16 @@ QString AudioPlayerMetaData::artist() {
     return AudioPlayer::instance()->metaData(Artist).toString();
 }
 
-int AudioPlayerMetaData::audioBitRate() {
-    return AudioPlayer::instance()->metaData(AudioBitRate).toInt();
+QString AudioPlayerMetaData::artistId() {
+    return AudioPlayer::instance()->metaData(ArtistId).toString();
 }
 
-QString AudioPlayerMetaData::audioBitRateString() {
-    return Utils::formatBytes(qint64(audioBitRate()))  + "/s";
+QString AudioPlayerMetaData::date() {
+    return AudioPlayer::instance()->metaData(Date).toString();
 }
 
-QString AudioPlayerMetaData::audioCodec() {
-    return AudioPlayer::instance()->metaData(AudioCodec).toString();
+QString AudioPlayerMetaData::description() {
+    return AudioPlayer::instance()->metaData(Description).toString();
 }
 
 qint64 AudioPlayerMetaData::duration() {
@@ -54,17 +54,29 @@ qint64 AudioPlayerMetaData::duration() {
 }
 
 QString AudioPlayerMetaData::durationString() {
-    return Utils::formatMSecs(duration());
+    return AudioPlayer::instance()->metaData(DurationString).toString();
+}
+
+QString AudioPlayerMetaData::format() {
+    return AudioPlayer::instance()->metaData(Format).toString();
 }
 
 QString AudioPlayerMetaData::genre() {
     return AudioPlayer::instance()->metaData(Genre).toString();
 }
 
+QString AudioPlayerMetaData::id() {
+    return AudioPlayer::instance()->metaData(Id).toString();
+}
+
 bool AudioPlayerMetaData::isAvailable() {
     return AudioPlayer::instance()->currentTrack() ? true : false;
 }
 
+bool AudioPlayerMetaData::isDownloadable() {
+    return AudioPlayer::instance()->metaData(Downloadable).toBool();
+}
+
 QUrl AudioPlayerMetaData::largeThumbnailUrl() {
     return AudioPlayer::instance()->metaData(LargeThumbnailUrl).toString();
 }
@@ -73,12 +85,28 @@ QUrl AudioPlayerMetaData::thumbnailUrl() {
     return AudioPlayer::instance()->metaData(ThumbnailUrl).toString();
 }
 
+QString AudioPlayerMetaData::service() {
+    return AudioPlayer::instance()->metaData(Service).toString();
+}
+
+qint64 AudioPlayerMetaData::size() {
+    return AudioPlayer::instance()->metaData(Size).toLongLong();
+}
+
+QString AudioPlayerMetaData::sizeString() {
+    return AudioPlayer::instance()->metaData(SizeString).toString();
+}
+
+QUrl AudioPlayerMetaData::streamUrl() {
+    return AudioPlayer::instance()->metaData(StreamUrl).toString();
+}
+
 QString AudioPlayerMetaData::title() {
     return AudioPlayer::instance()->metaData(Title).toString();
 }
 
-int AudioPlayerMetaData::trackNumber() {
-    return AudioPlayer::instance()->metaData(TrackNumber).toInt();
+QUrl AudioPlayerMetaData::url() {
+    return AudioPlayer::instance()->metaData(Url).toString();
 }
 
 AudioPlayer* AudioPlayer::self = 0;
@@ -94,20 +122,27 @@ AudioPlayer::AudioPlayer(QObject *parent) :
     m_shuffleIndex(0),
     m_repeat(false),
     m_shuffle(false),
+    m_sleepTimerDuration(30),
+    m_sleepTimerRemaining(0),
     m_stopAfterCurrentTrack(false),
-    m_status(Stopped)
-{    
+    m_status(Stopped),
+    m_metaDataSet(false)
+{
+    m_sleepTimer.setInterval(60000);
+    
     connect(m_player, SIGNAL(bufferStatusChanged(int)), this, SLOT(onBufferStatusChanged(int)));
     connect(m_player, SIGNAL(durationChanged(qint64)), this, SLOT(onDurationChanged(qint64)));
     connect(m_player, SIGNAL(error(QMediaPlayer::Error)), this, SLOT(onError(QMediaPlayer::Error)));
-    connect(m_player, SIGNAL(metaDataAvailableChanged(bool)), this, SLOT(onMetaDataAvailableChanged(bool)));
+    connect(m_player, SIGNAL(metaDataChanged()), this, SLOT(onMetaDataChanged()));
     connect(m_player, SIGNAL(mediaStatusChanged(QMediaPlayer::MediaStatus)),
             this, SLOT(onMediaStatusChanged(QMediaPlayer::MediaStatus)));
     connect(m_player, SIGNAL(positionChanged(qint64)), this, SIGNAL(positionChanged(qint64)));
     connect(m_player, SIGNAL(seekableChanged(bool)), this, SLOT(onSeekableChanged()));
     connect(m_player, SIGNAL(stateChanged(QMediaPlayer::State)), this, SLOT(onStateChanged(QMediaPlayer::State)));
+    connect(m_player, SIGNAL(volumeChanged(int)), this, SIGNAL(volumeChanged(int)));
     connect(this, SIGNAL(currentIndexChanged(int)), m_queue, SLOT(onCurrentIndexChanged()));
     connect(m_queue, SIGNAL(countChanged(int)), this, SIGNAL(queueCountChanged(int)));
+    connect(&m_sleepTimer, SIGNAL(timeout()), this, SLOT(onSleepTimerTimeout()));
 }
 
 AudioPlayer::~AudioPlayer() {
@@ -133,61 +168,47 @@ QVariant AudioPlayer::metaData(AudioPlayerMetaData::MetaData key) const {
     if (!track) {
         return QVariant();
     }
-#if QT_VERSION >= 0x050000
+    
     switch (key) {
     case AudioPlayerMetaData::Artist:
         return track->artist();
-    case AudioPlayerMetaData::AudioBitRate:
-        return m_player->metaData("AudioBitRate");
-    case AudioPlayerMetaData::AudioBitRateString:
-        return Utils::formatBytes(m_player->metaData("AudioBitRate").toLongLong()) + "/s";
-    case AudioPlayerMetaData::AudioCodec:
-        return m_player->metaData("AudioCodec");
-    case AudioPlayerMetaData::Duration:
-        return track->duration();
-    case AudioPlayerMetaData::DurationString:
-        return track->durationString();
-    case AudioPlayerMetaData::Genre:
-        return track->genre();
-    case AudioPlayerMetaData::LargeThumbnailUrl:
-        return track->largeThumbnailUrl();
-    case AudioPlayerMetaData::ThumbnailUrl:
-        return track->thumbnailUrl();
-    case AudioPlayerMetaData::Title:
-        return track->title();
-    case AudioPlayerMetaData::TrackNumber:
-        return m_player->metaData("TrackNumber");
-    default:
-        return QVariant();
-    }
-#else
-    switch (key) {
-    case AudioPlayerMetaData::Artist:
-        return track->artist();
-    case AudioPlayerMetaData::AudioBitRate:
-        return m_player->metaData(QtMultimediaKit::AudioBitRate);
-    case AudioPlayerMetaData::AudioBitRateString:
-        return Utils::formatBytes(m_player->metaData(QtMultimediaKit::AudioBitRate).toLongLong()) + "/s";
-    case AudioPlayerMetaData::AudioCodec:
-        return m_player->metaData(QtMultimediaKit::AudioCodec);
+    case AudioPlayerMetaData::ArtistId:
+        return track->id();
+    case AudioPlayerMetaData::Date:
+        return track->date();
+    case AudioPlayerMetaData::Description:
+        return track->description();
+    case AudioPlayerMetaData::Downloadable:
+        return track->isDownloadable();
     case AudioPlayerMetaData::Duration:
         return track->duration();
     case AudioPlayerMetaData::DurationString:
         return track->durationString();
+    case AudioPlayerMetaData::Format:
+        return track->format();
     case AudioPlayerMetaData::Genre:
         return track->genre();
+    case AudioPlayerMetaData::Id:
+        return track->id();
     case AudioPlayerMetaData::LargeThumbnailUrl:
         return track->largeThumbnailUrl();
     case AudioPlayerMetaData::ThumbnailUrl:
         return track->largeThumbnailUrl();
+    case AudioPlayerMetaData::Service:
+        return track->service();
+    case AudioPlayerMetaData::Size:
+        return track->size();
+    case AudioPlayerMetaData::SizeString:
+        return track->sizeString();
+    case AudioPlayerMetaData::StreamUrl:
+        return track->streamUrl();
     case AudioPlayerMetaData::Title:
         return track->title();
-    case AudioPlayerMetaData::TrackNumber:
-        return m_player->metaData(QtMultimediaKit::TrackNumber);
+    case AudioPlayerMetaData::Url:
+        return track->url();
     default:
         return QVariant();
     }
-#endif
 }
 
 int AudioPlayer::bufferStatus() const {
@@ -201,6 +222,8 @@ int AudioPlayer::currentIndex() const {
 void AudioPlayer::setCurrentIndex(int i, bool autoPlay) {
     if ((i >= 0) && (i < queueCount())) {
         m_index = i;
+        m_metaDataSet = false;
+        autoPlay = (autoPlay) || (isPlaying());
         stop();
         emit currentIndexChanged(i);
         emit metaDataChanged();
@@ -229,6 +252,11 @@ QString AudioPlayer::errorString() const {
 
 void AudioPlayer::setErrorString(const QString &e) {
     m_errorString = e;
+
+    if (!e.isEmpty()) {
+        Logger::log("AudioPlayer::error(). " + e);
+        emit error(e);
+    }
 }
 
 bool AudioPlayer::isPaused() const {
@@ -326,6 +354,55 @@ void AudioPlayer::setShuffleEnabled(bool s) {
     }
 }
 
+int AudioPlayer::sleepTimerDuration() const {
+    return m_sleepTimerDuration;
+}
+
+void AudioPlayer::setSleepTimerDuration(int d) {
+    if ((d != sleepTimerDuration()) && (d > 0)) {
+        m_sleepTimerDuration = d;
+        emit sleepTimerDurationChanged(d);
+    }
+}
+
+QString AudioPlayer::sleepTimerDurationString() const {
+    return Utils::formatMSecs(sleepTimerDuration() * 60000);
+}
+
+bool AudioPlayer::sleepTimerEnabled() const {
+    return m_sleepTimer.isActive();
+}
+
+void AudioPlayer::setSleepTimerEnabled(bool e) {
+    if (e != sleepTimerEnabled()) {
+        if (e) {
+            setSleepTimerRemaining(sleepTimerDuration());
+            m_sleepTimer.start();
+        }
+        else {
+            setSleepTimerRemaining(0);
+            m_sleepTimer.stop();
+        }
+        
+        emit sleepTimerEnabledChanged(e);
+    }
+}
+
+int AudioPlayer::sleepTimerRemaining() const {
+    return m_sleepTimerRemaining;
+}
+
+void AudioPlayer::setSleepTimerRemaining(int r) {
+    if ((r != sleepTimerRemaining()) && (r >= 0)) {
+        m_sleepTimerRemaining = r;
+        emit sleepTimerRemainingChanged(r);
+    }
+}
+
+QString AudioPlayer::sleepTimerRemainingString() const {
+    return Utils::formatMSecs(sleepTimerRemaining() * 60000);
+}
+
 bool AudioPlayer::stopAfterCurrentTrack() const {
     return m_stopAfterCurrentTrack;
 }
@@ -348,6 +425,14 @@ void AudioPlayer::setStatus(AudioPlayer::Status s) {
     }
 }
 
+int AudioPlayer::volume() const {
+    return m_player->volume();
+}
+
+void AudioPlayer::setVolume(int v) {
+    m_player->setVolume(v);
+}
+
 void AudioPlayer::addTrack(MKTrack *track) {
     m_queue->append(new MKTrack(track, m_queue));
     
@@ -450,7 +535,7 @@ void AudioPlayer::addLocalFile(const QString &fileName) {
             m_queue->append(trackFromUrl(QUrl::fromLocalFile(fileName)));
         }
         else {
-            Logger::log("AudioPlayer::addLocalFile(). Mimetype not supported for " + fileName);
+            Logger::log("AudioPlayer::addLocalFile(). Mimetype not supported for " + fileName, Logger::MediumVerbosity);
         }
     }
     else {
@@ -465,7 +550,7 @@ MKTrack* AudioPlayer::trackFromUrl(const QUrl &url) {
     track->setUrl(url);
     
     if (Utils::isLocalFile(url)) {
-        const QFileInfo info(url.toString());
+        const QFileInfo info(Utils::toLocalFile(url));
         const QDir dir = info.dir();
         track->setArtist(dir.dirName());
         track->setArtistId(dir.dirName());
@@ -494,7 +579,7 @@ int AudioPlayer::addUrl(const QUrl &url) {
     const int count = queueCount();
     
     if (Utils::isLocalFile(url)) {
-        addLocalFile(url.path());
+        addLocalFile(Utils::toLocalFile(url));
     }
     else {
         const QVariantMap resource = Resources::getResourceFromUrl(url.toString());
@@ -531,7 +616,7 @@ int AudioPlayer::addUrls(const QList<QUrl> &urls) {
     
     foreach (const QUrl &url, urls) {
         if (Utils::isLocalFile(url)) {
-            addLocalFile(url.path());
+            addLocalFile(Utils::toLocalFile(url));
         }
         else {
             const QVariantMap resource = Resources::getResourceFromUrl(url.toString());
@@ -610,7 +695,7 @@ int AudioPlayer::restoreQueue() {
     
     settings.endArray();
     setCurrentIndex(qMax(0, settings.value("currentIndex", 0).toInt()), false);
-    Logger::log(QString("AudioPlayer::restoreQueue(). %1 tracks restored").arg(size));
+    Logger::log(QString("AudioPlayer::restoreQueue(). %1 tracks restored").arg(size), Logger::LowVerbosity);
     return size;
 }
 
@@ -649,16 +734,16 @@ void AudioPlayer::saveQueue() {
 }
 
 void AudioPlayer::next(bool autoPlay) {
-    const bool playing = (autoPlay) || (isPlaying());
+    autoPlay = (autoPlay) || (isPlaying());
     
     if (shuffleEnabled()) {
         if (m_shuffleIndex < (m_shuffleOrder.size() - 1)) {
             m_shuffleIndex++;
-            setCurrentIndex(m_shuffleOrder.at(m_shuffleIndex), playing);
+            setCurrentIndex(m_shuffleOrder.at(m_shuffleIndex), autoPlay);
         }
     }
     else {
-        setCurrentIndex(currentIndex() + 1, playing);
+        setCurrentIndex(currentIndex() + 1, autoPlay);
     }
 }
 
@@ -734,22 +819,24 @@ int AudioPlayer::playUrls(const QList<QUrl> &urls) {
 }
 
 void AudioPlayer::previous(bool autoPlay) {
-    const bool playing = (autoPlay) || (isPlaying());
+    autoPlay = (autoPlay) || (isPlaying());
     
     if (shuffleEnabled()) {
         if (m_shuffleIndex > 0) {
             m_shuffleIndex--;
-            setCurrentIndex(m_shuffleOrder.at(m_shuffleIndex), playing);
+            setCurrentIndex(m_shuffleOrder.at(m_shuffleIndex), autoPlay);
         }
     }
     else {
-        setCurrentIndex(currentIndex() - 1, playing);
+        setCurrentIndex(currentIndex() - 1, autoPlay);
     }
 }
 
 void AudioPlayer::stop() {
     m_player->stop();
     m_player->setMedia(QMediaContent());
+    emit durationChanged(0);
+    emit seekableChanged(false);
     
     if ((m_soundcloudModel) && (m_soundcloudModel->status() == QSoundCloud::StreamsRequest::Loading)) {
         m_soundcloudModel->cancel();
@@ -797,7 +884,7 @@ void AudioPlayer::onBufferStatusChanged(int b) {
     emit bufferStatusChanged(b);
 }
 
-void AudioPlayer::onDurationChanged(qint64 d) {    
+void AudioPlayer::onDurationChanged(qint64 d) {
     if (d > 0) {
         if (MKTrack *track = currentTrack()) {
             track->setDuration(d);
@@ -810,10 +897,8 @@ void AudioPlayer::onDurationChanged(qint64 d) {
 }
 
 void AudioPlayer::onError(QMediaPlayer::Error e) {
-    Logger::log("AudioPlayer::onError(). Error: " + QString::number(QMediaPlayer::Error(e)) + ". Error string: "
-        + m_player->errorString());
-    
-    if (e != QMediaPlayer::NoError) {
+    // Symbian throws an error when setting empty media, so ignore the error in this case
+    if ((e != QMediaPlayer::NoError) && (!m_player->media().isNull())) {
         setErrorString(m_player->errorString());
         setStatus(Failed);
     }
@@ -851,32 +936,39 @@ void AudioPlayer::onMediaStatusChanged(QMediaPlayer::MediaStatus m) {
     }
 }
 
-void AudioPlayer::onMetaDataAvailableChanged(bool available) {
-    if (!available) {
-        emit metaDataChanged();
+void AudioPlayer::onMetaDataChanged() {
+    if (m_metaDataSet) {
         return;
     }
     
     MKTrack *track = currentTrack();
-    
+
     if (!track) {
+        emit metaDataChanged();
         return;
     }
     
     const QUrl url = track->url();
+    QUrl thumbnailUrl = track->thumbnailUrl();
     
     if (Utils::isLocalFile(url)) {
-        QUrl thumbnailUrl = track->thumbnailUrl();
-        
-        if (thumbnailUrl.isEmpty()) {        
-            thumbnailUrl = Utils::thumbnailUrlForFile(url.toLocalFile());
-            
+        if (thumbnailUrl.isEmpty()) {
+            thumbnailUrl = Utils::thumbnailUrlForFile(Utils::toLocalFile(url));
+
             if (!thumbnailUrl.isEmpty()) {
                 track->setThumbnailUrl(thumbnailUrl);
                 track->setLargeThumbnailUrl(thumbnailUrl);
             }
         }
     }
+    
+    if (m_player->availableMetaData().isEmpty()) {
+        Logger::log("AudioPlayer::onMetaDataChanged(). No metadata available", Logger::HighVerbosity);
+        emit metaDataChanged();
+        return;
+    }
+    
+    Logger::log("AudioPlayer::onMetaDataChanged(). Metadata is available", Logger::HighVerbosity);
 #if QT_VERSION >= 0x050000
     QVariant artist = m_player->metaData("ContributingArtist");
     
@@ -891,7 +983,13 @@ void AudioPlayer::onMetaDataAvailableChanged(bool available) {
     if (!artist.isNull()) {
         track->setArtist(artist.toString());
     }
-        
+
+    const QVariant date = m_player->metaData("Date");
+
+    if (!date.isNull()) {
+        track->setDate(date.toDateTime().toString("dd MMM yyyy"));
+    }
+
     QVariant genre = m_player->metaData("Genre");
     
     if (genre.isNull()) {
@@ -901,12 +999,32 @@ void AudioPlayer::onMetaDataAvailableChanged(bool available) {
     if (!genre.isNull()) {
         track->setGenre(genre.toString());
     }
+
+    const QVariant size = m_player->metaData("Size");
+
+    if (!size.isNull()) {
+        track->setSize(size.toLongLong());
+    }
     
     const QVariant title = m_player->metaData("Title");
     
     if (!title.isNull()) {
         track->setTitle(title.toString());
     }
+
+    if (thumbnailUrl.isEmpty()) {
+        QVariant cover = m_player->metaData("coverArtUrlLarge");
+
+        if (cover.isNull()) {
+            cover = m_player->metaData("coverArtUrlSmall");
+        }
+
+        if (!cover.isNull()) {
+            thumbnailUrl = cover.toString();
+            track->setThumbnailUrl(thumbnailUrl);
+            track->setLargeThumbnailUrl(thumbnailUrl);
+        }
+    }
 #else
     QVariant artist = m_player->metaData(QtMultimediaKit::ContributingArtist);
     
@@ -921,6 +1039,12 @@ void AudioPlayer::onMetaDataAvailableChanged(bool available) {
     if (!artist.isNull()) {
         track->setArtist(artist.toString());
     }
+
+    const QVariant date = m_player->metaData(QtMultimediaKit::Date);
+
+    if (!date.isNull()) {
+        track->setDate(date.toDateTime().toString("dd MMM yyyy"));
+    }
         
     QVariant genre = m_player->metaData(QtMultimediaKit::Genre);
     
@@ -931,13 +1055,34 @@ void AudioPlayer::onMetaDataAvailableChanged(bool available) {
     if (!genre.isNull()) {
         track->setGenre(genre.toString());
     }
+
+    const QVariant size = m_player->metaData(QtMultimediaKit::Size);
+
+    if (!size.isNull()) {
+        track->setSize(size.toLongLong());
+    }
     
     const QVariant title = m_player->metaData(QtMultimediaKit::Title);
     
     if (!title.isNull()) {
         track->setTitle(title.toString());
     }
-#endif    
+
+    if (thumbnailUrl.isEmpty()) {
+        QVariant cover = m_player->metaData(QtMultimediaKit::CoverArtUrlLarge);
+
+        if (cover.isNull()) {
+            cover = m_player->metaData(QtMultimediaKit::CoverArtUrlSmall);
+        }
+
+        if (!cover.isNull()) {
+            thumbnailUrl = cover.toString();
+            track->setThumbnailUrl(thumbnailUrl);
+            track->setLargeThumbnailUrl(thumbnailUrl);
+        }
+    }
+#endif
+    m_metaDataSet = true;
     emit metaDataChanged();
 }
 
@@ -978,6 +1123,15 @@ void AudioPlayer::onSeekableChanged() {
     emit seekableChanged(isSeekable());
 }
 
+void AudioPlayer::onSleepTimerTimeout() {
+    setSleepTimerRemaining(sleepTimerRemaining() - 1);
+    
+    if (sleepTimerRemaining() == 0) {
+        setSleepTimerEnabled(false);
+        stop();
+    }
+}
+
 void AudioPlayer::onSoundCloudModelStatusChanged(QSoundCloud::StreamsRequest::Status s) {
     switch (s) {
     case QSoundCloud::StreamsRequest::Loading:
